#include "SkillMemoryGui.h"

#include "gui_utils.h"

namespace armarx::skills::gui
{
    SkillMemoryGUI::SkillMemoryGUI(QTreeWidget* _skillExecutionTreeWidget,
                                   QLayout* _skillExecutionTreeWidgetParentLayout,
                                   QGroupBox* _skillGroupBox,
                                   QSplitter* _skillGroupBoxParentLayout,
                                   QGroupBox* _skillDetailGroupBox,
                                   QSplitter* _skillDetailGroupBoxParentLayout,
                                   QHBoxLayout* _updateWidgetLayout,

                                   QLayout* stopAllLayout,

                                   QWidget* _connectionStatusLabel,

                                   std::shared_ptr<SkillManagerWrapper> _memory)
    {
        Logging::setTag("SkillMemoryGui");

        // parameter check
        ARMARX_CHECK(_skillExecutionTreeWidgetParentLayout);
        ARMARX_CHECK(_skillGroupBoxParentLayout);
        ARMARX_CHECK(_skillDetailGroupBoxParentLayout);
        ARMARX_CHECK(_memory);

        ARMARX_CHECK(_skillExecutionTreeWidget);
        ARMARX_CHECK(_skillGroupBox);
        ARMARX_CHECK(_skillDetailGroupBox);
        ARMARX_CHECK(_updateWidgetLayout);
        ARMARX_CHECK(_connectionStatusLabel);
        ARMARX_CHECK(_memory);

        // setup memory
        this->memory = _memory;

        // Update widget
        this->updateWidgetLayout = _updateWidgetLayout;
        this->updateWidget = new PeriodicUpdateWidget();
        this->updateWidgetLayout->insertWidget(0, updateWidget);


        // replace skillExecutionTreeWidget
        this->skillExecutionTreeWidget = new SkillExecutionTreeWidget(memory);
        armarx::gui::replaceWidget(_skillExecutionTreeWidget,
                                   this->skillExecutionTreeWidget,
                                   _skillExecutionTreeWidgetParentLayout);

        // replace skillGroupBox
        this->skillGroupBox = new SkillGroupBox(memory);

        // replace skillDetailGroupBox
        this->skillDetailGroupBox = new SkillDetailGroupBox(memory);

        armarx::gui::clearSplitter(_skillGroupBoxParentLayout);
        _skillGroupBoxParentLayout->insertWidget(0, this->skillDetailGroupBox);
        _skillGroupBoxParentLayout->insertWidget(0, this->skillGroupBox);

        // setup stop all button
        stopAllButton = new QPushButton(QString::fromStdString(STOP_ALL_BUTTON_TEXT));
        stopAllLayout->addWidget(stopAllButton);

        this->connectionStatusLabel = new StatusLabel();
        armarx::gui::replaceWidget(_connectionStatusLabel,
                                   this->connectionStatusLabel,
                                   _connectionStatusLabel->parentWidget()->layout());

        setupUi();
    }

    void
    SkillMemoryGUI::setupUi()
    {
        stopAllButton->setStyleSheet("background-color: red");
        connectSignals();
    }

    void
    SkillMemoryGUI::connectSignals()
    {
        connect(skillGroupBox,
                &SkillGroupBox::updateSkillDetails,
                skillDetailGroupBox,
                &SkillDetailGroupBox::updateSkillDetails);

        // disconnect gui
        connect(this, &SkillMemoryGUI::disconnectGui, skillGroupBox, &SkillGroupBox::disconnectGui);
        connect(this,
                &SkillMemoryGUI::disconnectGui,
                skillDetailGroupBox,
                &SkillDetailGroupBox::disconnectGui);
        connect(this,
                &SkillMemoryGUI::disconnectGui,
                skillExecutionTreeWidget,
                &SkillExecutionTreeWidget::disconnectGui);
        connect(
            this, &SkillMemoryGUI::disconnectGui, updateWidget, &PeriodicUpdateWidget::stopTimer);

        // connect gui
        connect(this,
                &SkillMemoryGUI::connectGui,
                updateWidget,
                &PeriodicUpdateWidget::startTimerIfEnabled);
        connect(this, &SkillMemoryGUI::connectGui, skillGroupBox, &SkillGroupBox::connectGui);

        // timer
        connect(this->updateWidget,
                &PeriodicUpdateWidget::update,
                this->memory.get(),
                &SkillManagerWrapper::updateFromMemory);

        // updates
        connect(this->memory.get(),
                &SkillManagerWrapper::updateAvailable,
                this,
                &SkillMemoryGUI::updateGui);

        connect(this, &SkillMemoryGUI::updateGui, this->skillGroupBox, &SkillGroupBox::updateGui);

        connect(this,
                &SkillMemoryGUI::updateGui,
                this->skillExecutionTreeWidget,
                &SkillExecutionTreeWidget::updateGui);


        // stop all
        connect(stopAllButton,
                &QPushButton::clicked,
                memory.get(),
                &SkillManagerWrapper::stopAllExecutions);

        // status label
        connect(memory.get(),
                &SkillManagerWrapper::connectionUpdate,
                connectionStatusLabel,
                &StatusLabel::handleMessage);
    }
} // namespace armarx::skills::gui
